/*
 * Copyright (C) 2009 by Aiwota Programmer
 * aiwotaprog@tetteke.tk
 *
 * This file is part of Dialektos.
 *
 * Dialektos is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Dialektos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Dialektos.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board_view_column.hxx"

#include <sigc++/functors/mem_fun.h>
#include <glibmm/ustring.h>
#include <gtkmm/cellrenderer.h>
#include <gtkmm/treeviewcolumn.h>
#include <gtkmm/treemodel.h>

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/mpl/find.hpp>

#include "thread_list_model.hxx"


namespace dialektos {

namespace view_column {


Base::Base(const Glib::ustring& title) :
      Gtk::TreeViewColumn(title) {
  set_min_width(15);
  set_reorderable(true);
  set_resizable(true);
  set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
  set_clickable(true);
//    set_fixed_width(30); // specify in a derived class.

  Gtk::CellRendererText* cell = Gtk::manage(new Gtk::CellRendererText);
  pack_start(*cell, true);
  set_cell_data_func(*cell,
      sigc::mem_fun(*this, &Base::on_cell_data));
}

Base::~Base() {}
void Base::on_cell_data(Gtk::CellRenderer* cell,
    const Gtk::TreeModel::iterator& iter) {
  do_cell_data(cell, iter);
}

StartTime::StartTime() : Base("Since") {
  set_fixed_width(50);
}
StartTime::~StartTime() {}

void StartTime::do_cell_data(Gtk::CellRenderer* cell,
    const Gtk::TreeModel::iterator& iter) {

  model_column::ID::type value;
  iter->get_value(boost::mpl::find<ModelColumns::type,
      model_column::ID>::type::pos::value, value);

  try {
    std::time_t t = boost::lexical_cast<std::time_t>(value);
    boost::posix_time::ptime ptime = boost::posix_time::from_time_t(t);
    boost::posix_time::time_facet* facet =
      new boost::posix_time::time_facet("%Y/%m/%d %H:%M:%S");
    std::stringstream ss;
    ss.imbue(std::locale(ss.getloc(), facet));
    ss << ptime;
    value = ss.str();
  } catch (const std::exception& /*e*/) {
    value = "";
  }

  cell->set_property("text", value);
}


Average::Average() : Base("Average") {
  set_fixed_width(50);
}
Average::~Average() {}

void Average::do_cell_data(Gtk::CellRenderer* cell,
    const Gtk::TreeModel::iterator& iter) {
  on_cell_data_int<model_column::Average>(cell, iter);
}


Title::Title() : Base("Title") {
  set_fixed_width(200);
}
Title::~Title() {}
void Title::do_cell_data(Gtk::CellRenderer* cell,
    const Gtk::TreeModel::iterator& iter) {
  on_cell_data_string<model_column::Title>(cell, iter);
}


ResNum::ResNum() : Base("ResNum") {
  set_fixed_width(30);
}
ResNum::~ResNum() {}
void ResNum::do_cell_data(Gtk::CellRenderer* cell,
    const Gtk::TreeModel::iterator& iter) {
  on_cell_data_int<model_column::ResNum>(cell, iter);
}


Number::Number() : Base("No.") {
  set_fixed_width(30);
}
Number::~Number() {}
void Number::do_cell_data(Gtk::CellRenderer* cell,
    const Gtk::TreeModel::iterator& iter) {
  on_cell_data_int<model_column::Number>(cell, iter);
}


LineCount::LineCount() : Base("Read") {
  set_fixed_width(30);
}
LineCount::~LineCount() {}
void LineCount::do_cell_data(Gtk::CellRenderer* cell,
    const Gtk::TreeModel::iterator& iter) {
  on_cell_data_int<model_column::LineCount>(cell, iter);
}


} // namespace view_column

} // namespace dialektos
