/*
 * Copyright (C) 2009 by Aiwota Programmer
 * aiwotaprog@tetteke.tk
 *
 * This file is part of Dialektos.
 *
 * Dialektos is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Dialektos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Dialektos.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef BOARD_VIEW_COLUMN_HXX
#define BOARD_VIEW_COLUMN_HXX

#include <gtkmm/treeviewcolumn.h>
#include <glibmm/ustring.h>
#include <gtkmm/treemodel.h>
#include <gtkmm/cellrenderer.h>
#include <boost/mpl/find.hpp>
#include "thread_list_model.hxx"


namespace dialektos {

namespace view_column {


class Base : public Gtk::TreeViewColumn {
public:
  Base(const Glib::ustring& title);
  virtual ~Base();
protected:
  template <typename ModelColumnType>
  void on_cell_data_int(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter);
  template <typename ModelColumnType>
  void on_cell_data_string(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter);
private:
  virtual void do_cell_data(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter) = 0;
  void on_cell_data(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter);
};

template <typename ModelColumnType>
void Base::on_cell_data_int(Gtk::CellRenderer* cell,
    const Gtk::TreeModel::iterator& iter) {
  typename ModelColumnType::type value;
  iter->get_value(boost::mpl::find<model_column::List,
      ModelColumnType>::type::pos::value, value);
  if (value > 0) cell->set_property("text", value);
  else cell->set_property("text", Glib::ustring(""));
}
template <typename ModelColumnType>
void Base::on_cell_data_string(Gtk::CellRenderer* cell,
    const Gtk::TreeModel::iterator& iter) {

  typename ModelColumnType::type value;
  iter->get_value(boost::mpl::find<model_column::List,
      ModelColumnType>::type::pos::value, value);

  cell->set_property("text", value);
}


class StartTime: public Base {
public:
  StartTime();
  virtual ~StartTime();
private:
  virtual void do_cell_data(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter);
};

class Average: public Base {
public:
  Average();
  virtual ~Average();

private:
  virtual void do_cell_data(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter);
};


class Title: public Base {
public:
  Title();
  virtual ~Title();
private:
  virtual void do_cell_data(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter);
};


class ResNum: public Base {
public:
  ResNum();
  virtual ~ResNum();
private:
  virtual void do_cell_data(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter);
};


class Number: public Base {
public:
  Number();
  virtual ~Number();
private:
  virtual void do_cell_data(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter);
};

class LineCount: public Base {
public:
  LineCount();
  virtual ~LineCount();
private:
  virtual void do_cell_data(Gtk::CellRenderer* cell,
      const Gtk::TreeModel::iterator& iter);
};


} // namespace view_column


} // namespace dialektos


#endif
