/*
 * Copyright (C) 2009 by Aiwota Programmer
 * aiwotaprog@tetteke.tk
 *
 * This file is part of Dialektos.
 *
 * Dialektos is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Dialektos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Dialektos.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef BOARD_WINDOW_HXX
#define BOARD_WINDOW_HXX

#include <glibmm/refptr.h>
#include <gtkmm/treeview.h>
#include <gtkmm/scrolledwindow.h>
#include <boost/scoped_ptr.hpp>
#include <memory>
#include "application_framework.hxx"
#include "thread_list_model.hxx"
#include "http_get.hxx"
#include "board_subject_item.hxx"
#include "board_subject_idx.hxx"


namespace dialektos {

namespace bbs_detail {
  class Base;
}


class BoardWindow: public dialektos::ApplicationFrameWork {
public:
  static void create(std::auto_ptr<bbs_detail::Base> bbs);
  virtual ~BoardWindow();
protected:
  BoardWindow(std::auto_ptr<bbs_detail::Base> bbs);

  void on_action_file_open();

  virtual void on_action_view_refresh();
  virtual void on_action_view_stop();
private:
  virtual bool is_same(const bbs_detail::Base& bbs) const;
  virtual std::string get_uri() const;

  void on_row_activated(const Gtk::TreeModel::Path&,
      Gtk::TreeViewColumn*);
  void load();

  void merge_subject_txt(
      const std::vector<SubjectItem>&,
      boost::unordered_map<std::string, ModelColumns>&);
  void unmerge_subject_txt(
      boost::unordered_map<std::string, ModelColumns>&);

  void on_http_get_end(bool);
  void on_refresh_end(const http::StatusLine&, const http::Header&);

  void save_content(const http::Response&);

  Gtk::TreeView tree_view_;
  boost::scoped_ptr<bbs_detail::Base> bbs_;
  Gtk::ScrolledWindow scrolled_;
  Glib::RefPtr<ThreadListModel> tree_model_;
  boost::scoped_ptr<http::GetInThread> http_getter_;
  SubjectIdx idx_;
};


} // namespace dialektos

#endif
