/*
 * Copyright (C) 2009 by Aiwota Programmer
 * aiwotaprog@tetteke.tk
 *
 * This file is part of Dialektos.
 *
 * Dialektos is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Dialektos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Dialektos.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TEXT_ELEMENT_CHAR_SIZE_CACHE_HXX
#define TEXT_ELEMENT_CHAR_SIZE_CACHE_HXX

#include <glibmm/refptr.h>
#include <pangomm/layout.h>
#include <boost/array.hpp>
#include <boost/utility.hpp>


namespace dialektos {

namespace text_element {


class CharSizeCache : boost::noncopyable {
  typedef boost::array<int, 1<<16> ArrayType;
public:
  CharSizeCache();
  void initialize();
  void get_char_size(gunichar uch, Glib::RefPtr<Pango::Layout>& layout,
      double& width, double& height, bool bold);
  void get_char_size_from_cache(gunichar uch,
      double& width, double& height, bool bold) const;
private:
  ArrayType& get_width_cache_array(bool bold) {
    return bold ? bold_char_width_cache : char_width_cache;
  }
  ArrayType& get_height_cache_array(bool bold) {
    return bold ? bold_char_height_cache : char_height_cache;
  }
  const ArrayType& get_width_cache_array(bool bold) const {
    return bold ? bold_char_width_cache : char_width_cache;
  }
  const ArrayType& get_height_cache_array(bool bold) const {
    return bold ? bold_char_height_cache : char_height_cache;
  }

  ArrayType char_width_cache; // pango units
  ArrayType char_height_cache; // pango units
  ArrayType bold_char_width_cache; // pango units
  ArrayType bold_char_height_cache; // pango units
};


} // namespace text_element

} // namespace dialektos


#endif
